@@ -106,9 +106,16 @@ GNUNET_FS_GTK_add_meta_data_to_list_store (void *cls, const char *plugin_name,
   data_to_insert = GNUNET_FS_GTK_dubious_meta_to_utf8 (format, data, data_len);
   if (NULL == data_to_insert)
     return 0;
-  gtk_list_store_insert_with_values (ls, NULL, G_MAXINT, 0, type, 1, format,
-				     2, EXTRACTOR_metatype_to_string (type),
-				     3, data_to_insert, -1);
+  gtk_list_store_insert_with_values (ls, NULL, G_MAXINT,
+                                     GNUNET_GTK_FS_MAIN_WINDOW_META_DATA_MC_META_TYPE,
+                                     type,
+                                     GNUNET_GTK_FS_MAIN_WINDOW_META_DATA_MC_META_FORMAT,
+                                     format,
+                                     GNUNET_GTK_FS_MAIN_WINDOW_META_DATA_MC_META_TYPE_STRING,
+                                     EXTRACTOR_metatype_to_string (type),
+                                     GNUNET_GTK_FS_MAIN_WINDOW_META_DATA_MC_META_VALUE,
+                                     data_to_insert,
+                                     -1);
   GNUNET_free (data_to_insert); 
   return 0;
 }
@@ -208,7 +215,7 @@ GNUNET_FS_GTK_mmap_and_scan (const char *filename,
   void *ddata;
   int ret;
 
-  if (GNUNET_OK != GNUNET_DISK_file_size (filename, &fsize, GNUNET_YES))
+  if (GNUNET_OK != GNUNET_DISK_file_size (filename, &fsize, GNUNET_YES, GNUNET_YES))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -302,9 +309,11 @@ GNUNET_FS_GTK_get_description_from_metadata (
  * the appropriate action.
  *
  * @param uri the URI
+ * @param anonymity_level anonymity level to use
  */
 void
-GNUNET_FS_GTK_handle_uri (const struct GNUNET_FS_Uri *uri)
+GNUNET_FS_GTK_handle_uri (const struct GNUNET_FS_Uri *uri,
+			  guint anonymity_level)
 {
   GtkTreeIter iter;
   GtkTreeModel *namespace_treestore;
@@ -321,6 +330,7 @@ GNUNET_FS_GTK_handle_uri (const struct GNUNET_FS_Uri *uri)
     struct DownloadEntry *de;
 
     de = GNUNET_malloc (sizeof (struct DownloadEntry));
+    de->anonymity = anonymity_level;
     de->uri = GNUNET_FS_uri_dup (uri);
     GNUNET_FS_GTK_open_download_as_dialog (de);
     return;
@@ -350,16 +360,18 @@ GNUNET_FS_GTK_handle_uri (const struct GNUNET_FS_Uri *uri)
       return;
     }
     gtk_tree_model_get (namespace_treestore, &iter,
-			1, &nsid,
-			-1);
+                        GNUNET_GTK_FS_MAIN_WINDOW_SEARCH_NAMESPACE_MC_KEY,
+                        &nsid,
+                        -1);
     while ( ( (NULL == nsid) ||
 	      (0 != memcmp (nsid,
 			    &want,
 			    sizeof (GNUNET_HashCode))) ) &&
 	    (gtk_tree_model_iter_next (namespace_treestore, &iter)) )
       gtk_tree_model_get (namespace_treestore, &iter,
-			  1, &nsid,
-			  -1);
+                          GNUNET_GTK_FS_MAIN_WINDOW_SEARCH_NAMESPACE_MC_KEY,
+                          &nsid,
+                          -1);
     if ( (NULL == nsid) ||
 	 (0 != memcmp (nsid,
 		       &want,
@@ -373,7 +385,9 @@ GNUNET_FS_GTK_handle_uri (const struct GNUNET_FS_Uri *uri)
 				    (namespace_tree), &iter);
     treepath = gtk_tree_model_get_path (namespace_treestore, 
 					&iter);
-    if (GNUNET_GTK_get_tree_string (namespace_tree, treepath, 0, &value))
+    if (GNUNET_GTK_get_tree_string (namespace_tree, treepath,
+                                    GNUNET_GTK_FS_MAIN_WINDOW_SEARCH_NAMESPACE_MC_NAME,
+                                    &value))
       gtk_label_set_text (sel_namespace_label, value);
     gtk_tree_path_free (treepath);          
 
@@ -398,7 +412,9 @@ GNUNET_FS_GTK_handle_uri (const struct GNUNET_FS_Uri *uri)
       gtk_tree_selection_select_iter (gtk_tree_view_get_selection
 				      (namespace_tree), &iter);
       treepath = gtk_tree_model_get_path (namespace_treestore, &iter);
-      if (GNUNET_GTK_get_tree_string (namespace_tree, treepath, 0, &value))
+      if (GNUNET_GTK_get_tree_string (namespace_tree, treepath,
+                                      GNUNET_GTK_FS_MAIN_WINDOW_SEARCH_NAMESPACE_MC_NAME,
+                                      &value))
 	gtk_label_set_text (sel_namespace_label, value);
       gtk_tree_path_free (treepath);      
     }
@@ -417,4 +433,117 @@ GNUNET_FS_GTK_handle_uri (const struct GNUNET_FS_Uri *uri)
 }
 
 
+/* Largest rating value among all namespaces. INT_MIN means "undefined" */
+static int largest_namespace_rating = INT_MIN;
+
+
+/**
+ * Helper function for finding the largest namespace rating.
+ *
+ * @param cls closure
+ * @param pseudonym pseudonym hash
+ * @param md metadata container
+ * @param rating rating
+ * @return GNUNET_OK to keep iterating
+ */
+static int
+find_largest_namespace_rating_iterator (void *cls,
+    const GNUNET_HashCode *pseudonym, const char *name,
+    const char *unique_name,
+    const struct GNUNET_CONTAINER_MetaData *md, int rating)
+{
+  int *largest = cls;
+  if (*largest < rating)
+    *largest = rating;
+  return GNUNET_OK;
+}
+
+/**
+ * Finds largest namespace rating.
+ * Used to calculate a rating for newly discovered namespaces.
+ * Returns from cache, if possible.
+ *
+ * @return largest namespace rating. Might be negative and even. INT_MIN means
+ *         that no namespaces are known.
+ */
+int
+GNUNET_GTK_find_largest_namespace_rating ()
+{
+  if (largest_namespace_rating != INT_MIN)
+    return largest_namespace_rating;
+  (void) GNUNET_PSEUDONYM_list_all (GNUNET_FS_GTK_get_configuration (),
+      find_largest_namespace_rating_iterator, &largest_namespace_rating);
+  return largest_namespace_rating;
+}
+
+/**
+ * Sets largest namespace rating.
+ * Used to change cached largest namespace rating, when namespace list
+ * was changed in a way that is easy to track.
+ * If namespace list was changed in a way that makes it difficult to
+ * decide upon the new value, set new value to INT_MIN.
+ *
+ * @param new_value new value for the rating.
+ */
+void
+GNUNET_GTK_set_largest_namespace_rating (int new_value)
+{
+  largest_namespace_rating = new_value;
+}
+
+/**
+ * Converts a GtkTreeRowReference to a GtkTreeIter.
+ *
+ * @param rr row reference
+ * @param iter pointer to an iter structure to fill
+ * @return GNUNET_OK if iter was filled, GNUNET_SYSERR otherwise
+ */
+int
+GNUNET_GTK_get_iter_from_reference (GtkTreeRowReference *rr, GtkTreeIter *iter)
+{
+  int result = GNUNET_SYSERR;
+  if (rr != NULL)
+  {
+    if (gtk_tree_row_reference_valid (rr))
+    {
+      GtkTreePath *path;
+      GtkTreeModel *model;
+      path = gtk_tree_row_reference_get_path (rr);
+      model = gtk_tree_row_reference_get_model (rr);
+      if (path != NULL && model != NULL)
+      {
+        if (gtk_tree_model_get_iter (model,
+            iter, path))
+          result = GNUNET_OK;
+        gtk_tree_path_free (path);
+      }
+    }
+  }
+  return result;
+}
+
+/**
+ * Creates a GtkTreeRowReference from a GtkTreeIter.
+ *
+ * @param model a model to reference
+ * @param iter an iter that points to a row in the model
+ * @return newly created reference or NULL in case of error
+ */
+GtkTreeRowReference *
+GNUNET_GTK_get_reference_from_iter (GtkTreeModel *model, GtkTreeIter *iter)
+{
+  GtkTreeRowReference *result = NULL;
+  if (iter != NULL && model != NULL)
+  {
+    GtkTreePath *path = gtk_tree_model_get_path (model, iter);
+    if (path != NULL)
+    {
+      result = gtk_tree_row_reference_new (model, path);
+      gtk_tree_path_free (path);
+    }
+  }
+  return result;
+}
+
+
 /* end of gnunet-fs-gtk-common.c */
