@@ -37,6 +37,105 @@
 #define VERBOSE_PROGRESS GNUNET_NO
 
 
+/**
+ * Columns in the publish model.
+ */
+enum PUBLISH_ModelColumns
+  {
+    /**
+     * A gchararray.
+     */
+    PUBLISH_MC_FILESIZE = 0,
+
+    /**
+     * A gboolean.
+     */
+    PUBLISH_MC_DO_INDEX = 1,
+
+    /**
+     * A gchararray.
+     */
+    PUBLISH_MC_FILENAME = 2,
+
+    /**
+     * A guint.
+     */
+    PUBLISH_MC_ANONYMITY_LEVEL = 3,
+
+    /**
+     * A guint.
+     */
+    PUBLISH_MC_PRIORITY = 4,
+
+    /**
+     * A gpointer.
+     */
+    PUBLISH_MC_FILE_INFORMATION_STRUCT = 5,
+
+    /**
+     * A guint64.
+     */
+    PUBLISH_MC_EXPIRATION_TIME_ABSOLUTE = 6,
+
+    /**
+     * A guint.
+     */
+    PUBLISH_MC_REPLICATION_LEVEL = 7,
+  };
+
+
+/**
+ * Columns in the pseudonym model.
+ */
+enum PSEUDONYM_ModelColumns
+  {
+    /**
+     * A gchararray.
+     */
+    PSEUDONYM_MC_LOCAL_NAME = 0,
+
+    /**
+     * A gpointer.
+     */
+    PSEUDONYM_MC_NAMESPACE_HANDLE = 1,
+
+    /**
+     * A gchararray.
+     */
+    PSEUDONYM_MC_LAST_ID = 2,
+
+    /**
+     * A gchararray.
+     */
+    PSEUDONYM_MC_LAST_URI = 3,
+
+    /**
+     * A gpointer.
+     */
+    PSEUDONYM_MC_LAST_META = 4,
+
+    /**
+     * A gchararray.
+     */
+    PSEUDONYM_MC_NEXT_ID = 5,
+
+    /**
+     * A gchararray.
+     */
+    PSEUDONYM_MC_LAST_DESCRIPTION_FROM_META = 6,
+
+    /**
+     * A gboolean.
+     */
+    PSEUDONYM_MC_NEXT_ID_EDITABLE = 7,
+
+    /**
+     * A gboolean.
+     */
+    PSEUDONYM_MC_CURRENT_ID_EDITABLE = 8,
+  };
+
+
 /**
  * Context we create when we are scanning a directory.
  */
@@ -281,7 +380,8 @@ update_selectivity (struct MainPublishingDialogContext *ctx)
   ns_ok = GNUNET_YES;
   if (gtk_tree_selection_get_selected (ctx->pseudonym_selection, NULL, &iter))
   {
-    gtk_tree_model_get (ctx->pseudonym_treemodel, &iter, 2, &namespace_id, -1);
+    gtk_tree_model_get (ctx->pseudonym_treemodel, &iter,
+                        PSEUDONYM_MC_LAST_ID, &namespace_id, -1);
     if (namespace_id == NULL)
       ns_ok = GNUNET_NO;
     else
@@ -339,7 +439,8 @@ update_selectivity (struct MainPublishingDialogContext *ctx)
   is_dir = GNUNET_SYSERR;
   while (GNUNET_YES != gtk_tree_iter_equals (ctx->file_info_treemodel, &pred, &iter))
   {
-    gtk_tree_model_get (ctx->file_info_treemodel, &pred, 5, &fip, -1);
+    gtk_tree_model_get (ctx->file_info_treemodel, &pred,
+                        PUBLISH_MC_FILE_INFORMATION_STRUCT, &fip, -1);
     is_dir = GNUNET_FS_file_information_is_directory (fip);
     GNUNET_assert (gtk_tree_model_iter_next (ctx->file_info_treemodel, &pred));
   }
@@ -405,15 +506,17 @@ create_dir_at_iter (struct MainPublishingDialogContext *ctx,
       (GNUNET_FS_GTK_get_fs_handle (), row_reference, NULL, meta, bo, name);
   GNUNET_CONTAINER_meta_data_destroy (meta);
   gtk_tree_store_set (GTK_TREE_STORE (ctx->file_info_treemodel), pos, 
-		      0, MARKER_DIR_FILE_SIZE, 
-		      1, (gboolean) GNUNET_NO,
-                      2, name, 
-		      3, (guint) bo->anonymity_level, 
-		      4, (guint) bo->content_priority, 
-		      5, fi,
-		      6, (guint64) bo->expiration_time.abs_value,
-		      7, (guint) bo->replication_level,
-		      -1);
+                      PUBLISH_MC_FILESIZE, MARKER_DIR_FILE_SIZE, 
+                      PUBLISH_MC_DO_INDEX, (gboolean) GNUNET_NO,
+                      PUBLISH_MC_FILENAME, name, 
+                      PUBLISH_MC_ANONYMITY_LEVEL, (guint) bo->anonymity_level, 
+                      PUBLISH_MC_PRIORITY, (guint) bo->content_priority, 
+                      PUBLISH_MC_FILE_INFORMATION_STRUCT, fi,
+                      PUBLISH_MC_EXPIRATION_TIME_ABSOLUTE,
+                      (guint64) bo->expiration_time.abs_value,
+                      PUBLISH_MC_REPLICATION_LEVEL,
+                      (guint) bo->replication_level,
+                      -1);
   update_selectivity (ctx);
 }
 
@@ -453,25 +556,27 @@ copy_entry (struct MainPublishingDialogContext *ctx, GtkTreeModel * tm, GtkTreeI
     char *fsf;
 
     gtk_tree_model_get (tm, old,
-			0, &fsf, 
-			1, &do_index, 
-			2, &short_fn, 
-			3, &anonymity_level, 
-			4, &priority,
-			5, &fip, 
-			6, &expiration_time_abs,
-			7, &replication_level, 
-			-1);
-    gtk_tree_store_set (GTK_TREE_STORE (tm), newpos, 
-			0, fsf, 
-			1, do_index, 
-			2, short_fn, 
-			3, anonymity_level, 
-			4, priority,
-			5, fip, 
-			6, expiration_time_abs,
-			7, replication_level, 
-			-1);
+                        PUBLISH_MC_FILESIZE, &fsf, 
+                        PUBLISH_MC_DO_INDEX, &do_index,
+                        PUBLISH_MC_FILENAME, &short_fn,
+                        PUBLISH_MC_ANONYMITY_LEVEL, &anonymity_level,
+                        PUBLISH_MC_PRIORITY, &priority,
+                        PUBLISH_MC_FILE_INFORMATION_STRUCT, &fip,
+                        PUBLISH_MC_EXPIRATION_TIME_ABSOLUTE,
+                        &expiration_time_abs,
+                        PUBLISH_MC_REPLICATION_LEVEL, &replication_level,
+                        -1);
+    gtk_tree_store_set (GTK_TREE_STORE (tm), newpos,
+                        PUBLISH_MC_FILESIZE, fsf,
+                        PUBLISH_MC_DO_INDEX, do_index,
+                        PUBLISH_MC_FILENAME, short_fn,
+                        PUBLISH_MC_ANONYMITY_LEVEL, anonymity_level,
+                        PUBLISH_MC_PRIORITY, priority,
+                        PUBLISH_MC_FILE_INFORMATION_STRUCT, fip,
+                        PUBLISH_MC_EXPIRATION_TIME_ABSOLUTE,
+                        expiration_time_abs,
+                        PUBLISH_MC_REPLICATION_LEVEL, replication_level,
+                        -1);
     g_free (short_fn);
     g_free (fsf);
   }
@@ -548,8 +653,8 @@ GNUNET_GTK_master_publish_dialog_pseudonym_updates_renderer_edited_cb (GtkCellRe
     return;
   }
   gtk_tree_store_set (GTK_TREE_STORE (ctx->pseudonym_treemodel), &iter, 
-		      5, new_text, 
-		      -1);
+                      PSEUDONYM_MC_NEXT_ID, new_text, 
+                      -1);
   update_selectivity (ctx);
 }
 
@@ -578,8 +683,8 @@ GNUNET_GTK_master_publish_dialog_pseudonym_identifier_renderer_edited_cb (GtkCel
     return;
   }
   gtk_tree_store_set (GTK_TREE_STORE (ctx->pseudonym_treemodel), &iter, 
-		      2, new_text, 
-		      -1);
+                      PSEUDONYM_MC_LAST_ID, new_text, 
+                      -1);
   update_selectivity (ctx);
 }
 
@@ -878,7 +983,8 @@ GNUNET_GTK_master_publish_dialog_delete_button_clicked_cb (GtkWidget * dummy,
   GNUNET_assert (gtk_tree_selection_get_selected (ctx->file_info_selection, NULL, &iter));
 
   /* now delete the subtree */
-  gtk_tree_model_get (ctx->file_info_treemodel, &iter, 5, &fip, -1);
+  gtk_tree_model_get (ctx->file_info_treemodel, &iter,
+                      PUBLISH_MC_FILE_INFORMATION_STRUCT, &fip, -1);
   GNUNET_FS_file_information_destroy (fip, &free_fi_row_reference, NULL);
   GNUNET_FS_GTK_remove_treestore_subtree  (GTK_TREE_STORE (ctx->file_info_treemodel), 
 					   &iter);
@@ -1012,6 +1118,7 @@ add_item (struct AddDirClientContext *adcc,
   GtkTreeRowReference *row_reference;
   GtkTreePath *path;
   struct stat sbuf;
+  uint64_t fsize;
   
   if (0 != stat (item->filename,
 		 &sbuf))
@@ -1019,6 +1126,10 @@ add_item (struct AddDirClientContext *adcc,
     GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_ERROR, "stat", item->filename);
     return;
   }
+
+  if ((item->is_directory != GNUNET_YES) && (GNUNET_OK !=
+      GNUNET_DISK_file_size (item->filename, &fsize, GNUNET_YES, GNUNET_YES)))
+    return;
   gtk_tree_store_insert_after (ts, item_iter, parent, sibling);
   path = gtk_tree_model_get_path (GTK_TREE_MODEL (ts), item_iter);
   row_reference = gtk_tree_row_reference_new (GTK_TREE_MODEL (ts), path);
@@ -1050,17 +1161,22 @@ add_item (struct AddDirClientContext *adcc,
 						      item->meta, 
 						      adcc->directory_scan_do_index,
 						      &adcc->directory_scan_bo);
-    file_size_fancy = GNUNET_STRINGS_byte_size_fancy (sbuf.st_size);
+    file_size_fancy = GNUNET_STRINGS_byte_size_fancy (fsize);
   }
   gtk_tree_store_set (ts, item_iter, 
-		      0, file_size_fancy,
-		      1, (gboolean) adcc->directory_scan_do_index,
-		      2, item->short_filename,
-		      3, (guint) adcc->directory_scan_bo.anonymity_level,
-		      4, (guint) adcc->directory_scan_bo.content_priority,
-		      5, fi,
-		      6, (guint64) adcc->directory_scan_bo.expiration_time.abs_value,
-		      7, (guint) adcc->directory_scan_bo.replication_level, -1);
+                      PUBLISH_MC_FILESIZE, file_size_fancy,
+                      PUBLISH_MC_DO_INDEX,
+                      (gboolean) adcc->directory_scan_do_index,
+                      PUBLISH_MC_FILENAME, item->short_filename,
+                      PUBLISH_MC_ANONYMITY_LEVEL,
+                      (guint) adcc->directory_scan_bo.anonymity_level,
+                      PUBLISH_MC_PRIORITY,
+                      (guint) adcc->directory_scan_bo.content_priority,
+                      PUBLISH_MC_FILE_INFORMATION_STRUCT, fi,
+                      PUBLISH_MC_EXPIRATION_TIME_ABSOLUTE,
+                      (guint64) adcc->directory_scan_bo.expiration_time.abs_value,
+                      PUBLISH_MC_REPLICATION_LEVEL,
+                      (guint) adcc->directory_scan_bo.replication_level, -1);
   GNUNET_free (file_size_fancy);
 }
 
@@ -1548,13 +1664,15 @@ update_treeview_after_edit (void *cls, struct GNUNET_FS_FileInformation *fi,
 {
   struct EditPublishContext *epc = cls;
   
-  gtk_tree_store_set (GTK_TREE_STORE (epc->tm), &epc->iter, 
-		      1, *do_index,
-		      3, (guint) bo->anonymity_level, 
-		      4, (guint) bo->content_priority, 
-		      6, (guint64) bo->expiration_time.abs_value,
-		      7, (guint) bo->replication_level,			
-		      -1); 
+  gtk_tree_store_set (GTK_TREE_STORE (epc->tm), &epc->iter,
+                      PUBLISH_MC_DO_INDEX, *do_index,
+                      PUBLISH_MC_ANONYMITY_LEVEL, (guint) bo->anonymity_level,
+                      PUBLISH_MC_PRIORITY, (guint) bo->content_priority,
+                      PUBLISH_MC_EXPIRATION_TIME_ABSOLUTE,
+                      (guint64) bo->expiration_time.abs_value,
+                      PUBLISH_MC_REPLICATION_LEVEL,
+                      (guint) bo->replication_level,
+                      -1);
   return GNUNET_SYSERR;
 }
 
@@ -1602,9 +1720,9 @@ GNUNET_GTK_master_publish_dialog_edit_button_clicked_cb (GtkWidget * dummy,
     return;
   }
   gtk_tree_model_get (ctx->file_info_treemodel,
-		      &epc->iter,
-		      5, &epc->fip, 
-		      -1);
+                      &epc->iter,
+                      PUBLISH_MC_FILE_INFORMATION_STRUCT, &epc->fip,
+                      -1);
   GNUNET_FS_GTK_edit_publish_dialog (ctx->master_pubdialog, 
                                      epc->fip, 
 				     GNUNET_NO, 
@@ -1634,8 +1752,9 @@ get_file_information (GtkTreeModel * tm, GtkTreeIter * iter)
   struct GNUNET_FS_FileInformation *fic;
   GtkTreeIter child;
 
-  gtk_tree_model_get (tm, iter, 5, &fi, -1);
-  gtk_tree_store_set (GTK_TREE_STORE (tm), iter, 5, NULL, -1);
+  gtk_tree_model_get (tm, iter, PUBLISH_MC_FILE_INFORMATION_STRUCT, &fi, -1);
+  gtk_tree_store_set (GTK_TREE_STORE (tm), iter,
+                      PUBLISH_MC_FILE_INFORMATION_STRUCT, NULL, -1);
   GNUNET_assert (fi != NULL);
   if (gtk_tree_model_iter_children (tm, &child, iter))
   {
@@ -1665,9 +1784,9 @@ free_pseudonym_tree_store (GtkTreeModel * tm, GtkTreeIter * iter)
   struct GNUNET_FS_Namespace *ns;
 
   gtk_tree_model_get (tm, iter, 
-		      1, &ns, 
-		      4, &meta, 
-		      -1);
+                      PSEUDONYM_MC_NAMESPACE_HANDLE, &ns, 
+                      PSEUDONYM_MC_LAST_META, &meta, 
+                      -1);
   if (NULL != meta)
     GNUNET_CONTAINER_meta_data_destroy (meta);
   if (NULL != ns)
@@ -1695,7 +1814,7 @@ free_file_information_tree_store (GtkTreeModel * tm, GtkTreeIter * iter)
   GtkTreeIter child;
   struct GNUNET_FS_FileInformation *fip;
 
-  gtk_tree_model_get (tm, iter, 5, &fip, -1);
+  gtk_tree_model_get (tm, iter, PUBLISH_MC_FILE_INFORMATION_STRUCT, &fip, -1);
   if (NULL != fip)
     GNUNET_FS_file_information_destroy (fip, NULL, NULL);
   /* recursively clean up children */
@@ -1777,8 +1896,11 @@ GNUNET_GTK_master_publish_dialog_execute_button_clicked_cb (GtkButton * button,
   }
   if (TRUE == gtk_tree_selection_get_selected (ctx->pseudonym_selection, NULL, &iter))
   {
-    gtk_tree_model_get (ctx->pseudonym_treemodel, &iter, 1, &namespace, 2, &namespace_id, 5,
-			&namespace_uid, -1);
+    gtk_tree_model_get (ctx->pseudonym_treemodel, &iter,
+                        PSEUDONYM_MC_NAMESPACE_HANDLE, &namespace,
+                        PSEUDONYM_MC_LAST_ID, &namespace_id,
+                        PSEUDONYM_MC_NEXT_ID, &namespace_uid,
+                        -1);
   }
   else
   {
@@ -1914,14 +2036,23 @@ add_updateable_to_ts (void *cls, const char *last_id,
                                      GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST);
   /* FIXME-BUG-MAYBE: what if this put fails?  Not convinced it cannot... */
   desc = GNUNET_FS_GTK_get_description_from_metadata (last_meta, &desc_is_a_dup);
-  gtk_tree_store_insert_with_values (uc->ts, &iter, uc->parent, G_MAXINT, 0,
-                                     uc->namespace_name, 1,
-				     GNUNET_FS_namespace_dup (uc->ns), 2, last_id,
-                                     3, GNUNET_FS_uri_dup (last_uri), 4,
+  gtk_tree_store_insert_with_values (uc->ts, &iter, uc->parent, G_MAXINT,
+                                     PSEUDONYM_MC_LOCAL_NAME,
+                                     uc->namespace_name,
+                                     PSEUDONYM_MC_NAMESPACE_HANDLE,
+                                     GNUNET_FS_namespace_dup (uc->ns),
+                                     PSEUDONYM_MC_LAST_ID, last_id,
+                                     PSEUDONYM_MC_LAST_URI,
+                                     GNUNET_FS_uri_dup (last_uri),
+                                     PSEUDONYM_MC_LAST_META,
                                      GNUNET_CONTAINER_meta_data_duplicate
-                                     (last_meta), 5, "", 6, desc, 7,
+                                     (last_meta),
+                                     PSEUDONYM_MC_NEXT_ID, "",
+                                     PSEUDONYM_MC_LAST_DESCRIPTION_FROM_META,
+                                     desc,
+                                     PSEUDONYM_MC_NEXT_ID_EDITABLE,
                                      TRUE /* update editable (always) */ ,
-                                     8, FALSE
+                                     PSEUDONYM_MC_CURRENT_ID_EDITABLE, FALSE
                                      /* current not editable (only for top-level) */
                                      , -1);
   GNUNET_free_non_null (desc);
@@ -1937,12 +2068,21 @@ add_updateable_to_ts (void *cls, const char *last_id,
       (strlen (next_id) > 0))
   {
     /* add leaf */
-    gtk_tree_store_insert_with_values (uc->ts, &titer, &iter, G_MAXINT, 0,
-                                       uc->namespace_name, 1,
-				       GNUNET_FS_namespace_dup (uc->ns), 2,
-                                       next_id, 3, NULL, 4, NULL, 5, "", 6, "",
-                                       7, TRUE /* update editable (always) */ ,
-                                       8, FALSE
+    gtk_tree_store_insert_with_values (uc->ts, &titer, &iter, G_MAXINT,
+                                       PSEUDONYM_MC_LOCAL_NAME,
+                                       uc->namespace_name,
+                                       PSEUDONYM_MC_NAMESPACE_HANDLE,
+                                       GNUNET_FS_namespace_dup (uc->ns),
+                                       PSEUDONYM_MC_LAST_ID, next_id,
+                                       PSEUDONYM_MC_LAST_URI, NULL,
+                                       PSEUDONYM_MC_LAST_META, NULL,
+                                       PSEUDONYM_MC_NEXT_ID, "",
+                                       PSEUDONYM_MC_LAST_DESCRIPTION_FROM_META,
+                                       "",
+                                       PSEUDONYM_MC_NEXT_ID_EDITABLE,
+                                       TRUE /* update editable (always) */ ,
+                                       PSEUDONYM_MC_CURRENT_ID_EDITABLE,
+                                       FALSE
                                        /* current not editable (only for top-level) */
                                        , -1);
   }
@@ -1970,15 +2110,16 @@ add_namespace_to_ts (void *cls, const char *name, const GNUNET_HashCode * id)
   uc.ns = GNUNET_FS_namespace_create (GNUNET_FS_GTK_get_fs_handle (), name);
   uc.update_called = GNUNET_NO;
   gtk_tree_store_insert_with_values (ts, &iter, NULL, G_MAXINT, 
-				     0, name, 
-				     1, uc.ns, 
-				     2, NULL /* last-id */ ,
-                                     3, NULL /* last-uri (as string!) */ ,
-                                     4, NULL /* meta */ ,
-                                     5, NULL /* next-ID */ ,
-                                     6, NULL /* last-description */ ,
-                                     7, TRUE /* update editable */ ,
-                                     8, TRUE /* current editable */ ,
+                                     PSEUDONYM_MC_LOCAL_NAME, name,
+                                     PSEUDONYM_MC_NAMESPACE_HANDLE, uc.ns,
+                                     PSEUDONYM_MC_LAST_ID, NULL,
+                                     PSEUDONYM_MC_LAST_URI, NULL,
+                                     PSEUDONYM_MC_LAST_META, NULL,
+                                     PSEUDONYM_MC_NEXT_ID, NULL,
+                                     PSEUDONYM_MC_LAST_DESCRIPTION_FROM_META,
+                                     NULL,
+                                     PSEUDONYM_MC_NEXT_ID_EDITABLE, TRUE,
+                                     PSEUDONYM_MC_CURRENT_ID_EDITABLE, TRUE,
                                      -1);
   uc.seen = GNUNET_CONTAINER_multihashmap_create (128);
   GNUNET_FS_namespace_list_updateable (uc.ns, NULL, &add_updateable_to_ts, &uc);
