@@ -29,6 +29,99 @@
 #include "metatypes.c"
 
 
+/**
+ * Columns in the publication metadata model.
+ */
+enum PUBLISH_MetadataModelColumns
+  {
+    /**
+     * A guint.
+     */
+    PUBLISH_METADATA_MC_TYPE = 0,
+
+    /**
+     * A guint.
+     */
+    PUBLISH_METADATA_MC_FORMAT = 1,
+
+    /**
+     * A gchararray.
+     */
+    PUBLISH_METADATA_MC_TYPE_NAME = 2,
+
+    /**
+     * A gchararray.
+     */
+    PUBLISH_METADATA_MC_VALUE = 3,
+
+    /**
+     * A gchararray.
+     */
+    PUBLISH_METADATA_MC_DESCRIPTION = 4,
+  };
+
+
+/**
+ * Columns in the publication metadata types model.
+ */
+enum PUBLISH_MetadataTypesModelColumns
+  {
+    /**
+     * A gint.
+     */
+    PUBLISH_METADATA_TYPES_MC_TYPE = 0,
+
+    /**
+     * A gint.
+     */
+    PUBLISH_METADATA_TYPES_MC_FORMAT = 1,
+
+    /**
+     * A gchararray.
+     */
+    PUBLISH_METADATA_TYPES_MC_TYPE_NAME = 2,
+
+    /**
+     * A gchararray.
+     */
+    PUBLISH_METADATA_TYPES_MC_DESCRIPTION = 3,
+  };
+
+
+/**
+ * Columns in the publication types model.
+ */
+enum PUBLISH_TypesModelColumns
+  {
+    /**
+     * A gint.
+     */
+    PUBLISH_TYPES_MC_TYPE = 0,
+
+    /**
+     * A gchararray.
+     */
+    PUBLISH_TYPES_MC_TYPE_NAME = 1,
+  };
+
+
+/**
+ * Columns in the publication keywords model.
+ */
+enum PUBLISH_KeywordsModelColumns
+  {
+    /**
+     * A gchararray.
+     */
+    PUBLISH_TYPES_MC_KEYWORD = 0,
+
+    /**
+     * A gboolean.
+     */
+    PUBLISH_TYPES_MC_ADDED = 1,
+  };
+
+
 /**
  * Internal state kept for each "edit" dialog where the user can edit
  * publishing information for a file.
@@ -215,12 +308,18 @@ change_metatypes (struct EditPublicationDialogContext *ctx, gint code)
   for (i = 0; types[code][i] != EXTRACTOR_METATYPE_RESERVED; i++)  
     if ( (types[code][i] < max_type) && (types[code][i] > 0) )
       gtk_list_store_insert_with_values (ctx->metatypes_liststore, 
-					 &iter, G_MAXINT, 
-					 0, types[code][i], 
-					 1, EXTRACTOR_METAFORMAT_UTF8, 
-					 2, EXTRACTOR_metatype_to_string (types [code][i]), 
-					 3, EXTRACTOR_metatype_to_description (types[code][i]), 
-					 -1);  
+                                         &iter, G_MAXINT, 
+                                         PUBLISH_METADATA_TYPES_MC_TYPE,
+                                         types[code][i], 
+                                         PUBLISH_METADATA_TYPES_MC_FORMAT,
+                                         EXTRACTOR_METAFORMAT_UTF8, 
+                                         PUBLISH_METADATA_TYPES_MC_TYPE_NAME,
+                                         EXTRACTOR_metatype_to_string
+                                         (types [code][i]),
+                                         PUBLISH_METADATA_TYPES_MC_DESCRIPTION,
+                                         EXTRACTOR_metatype_to_description
+                                         (types[code][i]), 
+                                         -1);  
 }
 
 
@@ -241,7 +340,8 @@ GNUNET_GTK_edit_publication_type_combo_changed_cb (GtkComboBox * widget,
 
   if (! gtk_combo_box_get_active_iter (widget, &iter))
     return;
-  gtk_tree_model_get (GTK_TREE_MODEL (ctx->pubtypes_liststore), &iter, 0, &code, -1);
+  gtk_tree_model_get (GTK_TREE_MODEL (ctx->pubtypes_liststore), &iter,
+                      PUBLISH_TYPES_MC_TYPE, &code, -1);
   change_metatypes (ctx, code);
 }
 
@@ -301,11 +401,13 @@ GNUNET_GTK_edit_publication_metadata_tree_view_type_renderer_edited_cb (GtkCellR
   }
   type_id = ctx->meta_combo_selected_type_id;
   ctx->meta_combo_selected_type_id = -1;
-  gtk_list_store_set (ctx->meta_liststore, &iter, 
-		      0, type_id, 
-		      1, EXTRACTOR_METAFORMAT_UTF8, 
-		      2, EXTRACTOR_metatype_to_string (type_id),
-		      4, EXTRACTOR_metatype_to_description (type_id),
+  gtk_list_store_set (ctx->meta_liststore, &iter,
+                      PUBLISH_METADATA_MC_TYPE, type_id,
+                      PUBLISH_METADATA_MC_FORMAT, EXTRACTOR_METAFORMAT_UTF8,
+                      PUBLISH_METADATA_MC_TYPE_NAME,
+                      EXTRACTOR_metatype_to_string (type_id),
+                      PUBLISH_METADATA_MC_DESCRIPTION,
+                      EXTRACTOR_metatype_to_description (type_id),
                       -1);
 }
 
@@ -334,8 +436,8 @@ GNUNET_GTK_edit_publication_metadata_tree_view_type_renderer_changed_cb (GtkCell
   g_object_get (combo,
 		"model", &combo_model, NULL);
   gtk_tree_model_get (combo_model, new_iter,
-		      0, &type_id,
-		      -1);
+                      PUBLISH_METADATA_MC_TYPE, &type_id,
+                      -1);
   ctx->meta_combo_selected_type_id = type_id;
 }
 
@@ -373,7 +475,7 @@ GNUNET_GTK_edit_publication_metadata_tree_view_value_renderer_edited_cb (GtkCell
   }
 
   gtk_tree_model_get (GTK_TREE_MODEL (ctx->meta_liststore), &iter, 
-		      0, &metatype, -1);
+                      PUBLISH_METADATA_MC_TYPE, &metatype, -1);
   if (metatype == EXTRACTOR_METATYPE_GNUNET_ORIGINAL_FILENAME) 
   {
     /* apply filename rules */
@@ -413,8 +515,8 @@ GNUNET_GTK_edit_publication_metadata_tree_view_value_renderer_edited_cb (GtkCell
     avalue = NULL;
   }
   gtk_list_store_set (ctx->meta_liststore, &iter, 
-		      3, ivalue, 
-		      -1);
+                      PUBLISH_METADATA_MC_VALUE, ivalue, 
+                      -1);
   GNUNET_free_non_null (avalue);
 }
 
@@ -434,13 +536,16 @@ GNUNET_GTK_edit_publication_add_button_clicked_cb (GtkButton * button,
   GtkTreeIter iter;
 
   gtk_list_store_insert_with_values (ctx->meta_liststore,
-				     &iter, 0,
-				     0, 0,
-				     1, EXTRACTOR_METAFORMAT_UTF8, 
-				     2, _("Select a type"), 
-				     3, _("Specify a value"), 
-				     4, NULL, 
-				     -1);
+                                     &iter, 0,
+                                     PUBLISH_METADATA_MC_TYPE, 0,
+                                     PUBLISH_METADATA_MC_FORMAT,
+                                     EXTRACTOR_METAFORMAT_UTF8,
+                                     PUBLISH_METADATA_MC_TYPE_NAME,
+                                     _("Select a type"),
+                                     PUBLISH_METADATA_MC_VALUE,
+                                     _("Specify a value"),
+                                     PUBLISH_METADATA_MC_DESCRIPTION, NULL,
+                                     -1);
 }
 
 
@@ -608,10 +713,10 @@ GNUNET_GTK_edit_publication_keyword_list_add_button_clicked_cb (GtkButton *
   if (strlen (keyword) == 0)
     return;
   gtk_list_store_insert_with_values (ctx->keywords_liststore, 
-				     &iter, G_MAXINT,
-				     0, keyword,
-				     1, TRUE,
-				     -1);
+                                     &iter, G_MAXINT,
+                                     PUBLISH_TYPES_MC_KEYWORD, keyword,
+                                     PUBLISH_TYPES_MC_ADDED, TRUE,
+                                     -1);
   gtk_entry_set_text (ctx->keyword_entry, "");
   update_confirm_sensitivity (ctx);
 }
@@ -731,7 +836,11 @@ preserve_meta_items (void *cls, const char *plugin_name,
     {
       do
       {
-        gtk_tree_model_get (GTK_TREE_MODEL (ctx->meta_liststore), &iter, 0, &ntype, 1, &nformat, 3, &value, -1);
+        gtk_tree_model_get (GTK_TREE_MODEL (ctx->meta_liststore), &iter,
+                            PUBLISH_METADATA_MC_TYPE, &ntype,
+                            PUBLISH_METADATA_MC_FORMAT, &nformat,
+                            PUBLISH_METADATA_MC_VALUE, &value,
+                            -1);
         if ((ntype == type) && (nformat == format) &&
             (0 == strcmp (value, data)))
         {
@@ -809,7 +918,9 @@ file_information_update (void *cls, struct GNUNET_FS_FileInformation *fi,
     {
       gchar *value;
 
-      gtk_tree_model_get (GTK_TREE_MODEL (ctx->keywords_liststore), &iter, 0, &value, -1);
+      gtk_tree_model_get (GTK_TREE_MODEL (ctx->keywords_liststore), &iter,
+                          PUBLISH_TYPES_MC_KEYWORD, &value,
+                          -1);
       if (NULL == *uri)
 	*uri = GNUNET_FS_uri_ksk_create_from_args (1, (const char **) &value);
       else
@@ -840,7 +951,11 @@ file_information_update (void *cls, struct GNUNET_FS_FileInformation *fi,
       guint nformat;
       gchar *value;
 
-      gtk_tree_model_get (GTK_TREE_MODEL (ctx->meta_liststore), &iter, 0, &ntype, 1, &nformat, 3, &value, -1);
+      gtk_tree_model_get (GTK_TREE_MODEL (ctx->meta_liststore), &iter,
+                          PUBLISH_METADATA_MC_TYPE, &ntype,
+                          PUBLISH_METADATA_MC_FORMAT, &nformat,
+                          PUBLISH_METADATA_MC_VALUE, &value,
+                          -1);
       if (ntype > 0)
       {
         GNUNET_CONTAINER_meta_data_insert (meta, "<user>", ntype, nformat,
@@ -941,8 +1056,8 @@ add_keyword (void *cls, const char *keyword, int is_mandatory)
 
   ls = GTK_LIST_STORE (cls);
   gtk_list_store_insert_with_values (ls, &iter, G_MAXINT,
-				     0, keyword, 
-				     1, FALSE,
+                                     PUBLISH_TYPES_MC_KEYWORD, keyword,
+                                     PUBLISH_TYPES_MC_ADDED, FALSE,
                                      -1);
   return GNUNET_OK;
 }
@@ -1154,7 +1269,8 @@ GNUNET_FS_GTK_edit_publish_dialog (GtkWindow * parent,
                      (ctx->builder, "GNUNET_GTK_edit_publication_type_combo"));
   if (gtk_combo_box_get_active_iter (pubtypes_combo, &iter))
   {
-    gtk_tree_model_get (GTK_TREE_MODEL (ctx->pubtypes_liststore), &iter, 0, &code, -1);
+    gtk_tree_model_get (GTK_TREE_MODEL (ctx->pubtypes_liststore), &iter,
+                        PUBLISH_TYPES_MC_TYPE, &code, -1);
     change_metatypes (ctx, 0);
   }
   else
