@@ -25,7 +25,6 @@
  */
 #include "gnunet_gtk.h"
 
-#define DEBUG_EVENTLOOP GNUNET_NO
 
 /**
  * Initial size of our poll array cache.
@@ -42,9 +41,9 @@ struct GNUNET_GTK_MainLoop
 {
 
   /**
-   * Our configuration.
+   * Our configuration (includes defaults from gnunet-gtk/config.d/ and gnunet/config.d/)
    */
-  const struct GNUNET_CONFIGURATION_Handle *cfg;
+  struct GNUNET_CONFIGURATION_Handle *cfg;
 
   /**
    * Name of the glade file for the main window
@@ -191,6 +190,30 @@ GNUNET_GTK_main_loop_quit (struct GNUNET_GTK_MainLoop *ml)
   }
 }
 
+/**
+ * Get the builder from the main window.
+ *
+ * @param ml handle to the main loop
+ * @return NULL on error, otherwise the builder
+ */
+GtkBuilder *
+GNUNET_GTK_main_loop_get_builder (struct GNUNET_GTK_MainLoop *ml)
+{
+  return ml->builder;
+}
+
+int
+GNUNET_GTK_main_loop_build_window (struct GNUNET_GTK_MainLoop *ml, gpointer data)
+{
+  ml->builder = GNUNET_GTK_get_new_builder (ml->main_window_file, data);
+  if (ml->builder == NULL)
+  {
+    GNUNET_GTK_main_loop_quit (ml);
+    return GNUNET_SYSERR;
+  }
+  return GNUNET_OK;
+}
+
 
 /**
  * Obtain the name of the configuration file that is being used.
@@ -413,11 +436,9 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
 
             if (error_code == ERROR_IO_PENDING)
             {
-#if DEBUG_EVENTLOOP
               GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                           "Adding the pipe's 0x%x overlapped event to the array as %d\n",
-                          fh->h, nhandles);
-#endif
+                          fh->h, fd_counter);
               ml->cached_poll_array[fd_counter].fd =
                   (intptr_t) fh->oOverlapRead->hEvent;
               /* On W32 .events makes no sense - g_poll will just OR its
@@ -441,11 +462,9 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
           }
           else
           {
-#if DEBUG_EVENTLOOP
             GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                         "Adding the read ready event to the array as %d\n",
-                        nhandles);
-#endif
+                        fd_counter);
             ml->cached_poll_array[fd_counter].fd =
                 (intptr_t) ml->hEventReadReady;
             ml->cached_poll_array[fd_counter].events = G_IO_IN;
@@ -498,11 +517,9 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
 
     if ((rfds != NULL) && (rfds->sds.fd_count > 0))
     {
-#if DEBUG_EVENTLOOP
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                   "Adding the socket read event to the array as %d\n",
                   fd_counter);
-#endif
       ml->cached_poll_array[fd_counter].fd = (intptr_t) ml->hEventRead;
       ml->cached_poll_array[fd_counter].events = G_IO_IN;
       for (i = 0; i < rfds->sds.fd_count; i++)
@@ -515,11 +532,9 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
     {
       int wakeup = 0;
 
-#if DEBUG_EVENTLOOP
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                   "Adding the socket write event to the array as %d\n",
                   fd_counter);
-#endif
       ml->cached_poll_array[fd_counter].fd = (intptr_t) ml->hEventWrite;
       ml->cached_poll_array[fd_counter].events = G_IO_OUT;
       for (ui = 0; ui < wfds->sds.fd_count; ui++)
@@ -529,11 +544,9 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
 
         status = send (wfds->sds.fd_array[ui], NULL, 0, 0);
         error = GetLastError ();
-#if DEBUG_EVENTLOOP
         GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                     "pre-send to the socket %u returned %d (%u)\n", ui, status,
                     error);
-#endif
         if (status == 0 || (error != WSAEWOULDBLOCK && error != WSAENOTCONN))
           wakeup = 1;
         WSAEventSelect (wfds->sds.fd_array[ui], ml->hEventWrite,
@@ -546,11 +559,9 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
     }
     if ((efds != NULL) && (efds->sds.fd_count > 0))
     {
-#if DEBUG_EVENTLOOP
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                   "Adding the socket error event to the array as %d\n",
                   fd_counter);
-#endif
       ml->cached_poll_array[fd_counter].fd = (intptr_t) ml->hEventException;
       ml->cached_poll_array[fd_counter].events = G_IO_ERR;
       for (ui = 0; ui < efds->sds.fd_count; ui++)
@@ -591,24 +602,17 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
 #if WINDOWS
   if (pre_ret > 0)
   {
-#if DEBUG_EVENTLOOP
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "pre_ret is %d, setting delay to 0\n",
                 pre_ret);
-#endif
     delay = 0;
   }
 #endif
 
-#if DEBUG_EVENTLOOP
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "We have %d of our FDs and %d of GMC ones, going to wait %6dms\n",
               fd_counter, need_gfds, delay);
-#endif
-
   poll_result = g_poll (ml->cached_poll_array, fd_counter + need_gfds, delay);
-#if DEBUG_EVENTLOOP
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "g_poll returned : %d\n", poll_result);
-#endif
   if (-1 == poll_result)
     return GNUNET_SYSERR;
 
@@ -668,18 +672,14 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
     select_ret = select (max_nfds, &aread, &awrite, &aexcept, &tvslice);
     if (select_ret == -1)
       select_ret = 0;
-#if DEBUG_EVENTLOOP
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "select() returned %d\n", select_ret);
-#endif
   }
   if (always_ready_write_fd >= 0 &&
       ml->cached_poll_array[always_ready_write_fd].revents & G_IO_OUT)
   {
     GNUNET_CONTAINER_slist_append (ml->handles_write, wfds->handles);
     result += GNUNET_CONTAINER_slist_count (ml->handles_write);
-#if DEBUG_EVENTLOOP
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Added write pipe\n");
-#endif
   }
   for (i = 0; i < read_handles; i++)
   {
@@ -693,11 +693,9 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
     bret =
         PeekNamedPipe (ml->read_array[i]->h, NULL, 0, NULL, &waitstatus, NULL);
     error = GetLastError ();
-#if DEBUG_EVENTLOOP
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Peek at read pipe %d (0x%x) returned %d (%d bytes available) GLE %u\n",
                 i, ml->read_array[i]->h, bret, waitstatus, error);
-#endif
     if (bret == 0 || (ml->cached_poll_array[i].revents & G_IO_ERR))
     {
       if (efds != NULL)
@@ -728,16 +726,12 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
                                 ml->read_array[i],
                                 sizeof (struct GNUNET_DISK_FileHandle));
     result += 1;
-#if DEBUG_EVENTLOOP
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Added read Pipe 0x%x (0x%x)\n",
                 ml->read_array[i], ml->read_array[i]->h);
-#endif
   }
   waitstatus = WaitForSingleObject (ml->hEventWrite, 0);
-#if DEBUG_EVENTLOOP
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Wait for the write event returned %d\n",
               waitstatus);
-#endif
   if (waitstatus == WAIT_OBJECT_0)
   {
     for (ui = 0; ui < wfds->sds.fd_count; ui++)
@@ -752,11 +746,9 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
 
       status = send (wfds->sds.fd_array[ui], NULL, 0, 0);
       error = GetLastError ();
-#if DEBUG_EVENTLOOP
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                   "send to the socket %u returned %d (%u)\n", ui, status,
                   error);
-#endif
       if (status == 0 || (error != WSAEWOULDBLOCK && error != WSAENOTCONN) ||
           (status == -1 && gso_result == 0 && error == WSAENOTCONN &&
            so_error == WSAECONNREFUSED))
@@ -833,6 +825,9 @@ run_main_loop (void *cls, char *const *args, const char *cfgfile,
                const struct GNUNET_CONFIGURATION_Handle *cfg)
 {
   struct GNUNET_GTK_MainLoop *ml = cls;
+  struct GNUNET_CONFIGURATION_Handle *gcfg;
+  char *baseconfig;
+  char *ipath;
   int argc;
 
   /* command-line processing for Gtk arguments */
@@ -840,16 +835,24 @@ run_main_loop (void *cls, char *const *args, const char *cfgfile,
   while (args[argc] != NULL)
     argc++;
   gtk_init (&argc, (char ***) &args);
+  gcfg = GNUNET_CONFIGURATION_create ();
 
-  /* setup main context */
-  ml->builder = GNUNET_GTK_get_new_builder (ml->main_window_file, NULL);
-  if (ml->builder == NULL)
+  ipath = GNUNET_GTK_installation_get_path (GNUNET_OS_IPK_DATADIR);
+  if (ipath == NULL)
+  {
+    GNUNET_break (0);
     return;
+  }
+  GNUNET_asprintf (&baseconfig, "%s%s", ipath, "config.d");
+  GNUNET_free (ipath);
+  (void) GNUNET_CONFIGURATION_load_from (gcfg, baseconfig);
+  GNUNET_free (baseconfig);
+  (void) GNUNET_CONFIGURATION_load (gcfg, cfgfile);
   ml->rs = GNUNET_NETWORK_fdset_create ();
   ml->ws = GNUNET_NETWORK_fdset_create ();
   ml->gml = g_main_loop_new (NULL, TRUE);
   ml->gmc = g_main_loop_get_context (ml->gml);
-  ml->cfg = cfg;
+  ml->cfg = gcfg;
   ml->cfgfile = GNUNET_strdup (cfgfile);
 #if WINDOWS
   ml->hEventRead = CreateEvent (NULL, TRUE, FALSE, NULL);
@@ -918,6 +921,8 @@ GNUNET_GTK_main_loop_start (const char *binary_name, const char *binary_help,
     g_object_unref (G_OBJECT (ml.builder));
   if (NULL != ml.gml)
     g_main_loop_unref (ml.gml);
+  GNUNET_CONFIGURATION_destroy (ml.cfg);
+  ml.cfg = NULL;
   GNUNET_free_non_null (ml.cfgfile);
   return ret;
 }
