@@ -23,6 +23,7 @@
  * @brief configuration details
  * @author Christian Grothoff
  */
+#include "gnunet-setup.h"
 #include "gnunet-setup-options.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <gdk/gdkkeysyms.h>
@@ -605,7 +606,9 @@ GNUNET_setup_treeview_key_press_event_cb (GtkTreeView * tv, GdkEventKey * event,
   sel = gtk_tree_view_get_selection (tv);
   if (TRUE != gtk_tree_selection_get_selected (sel, &tm, &iter))
     return FALSE;
-  gtk_tree_model_get (tm, &iter, 1, &editable, -1);
+  gtk_tree_model_get (tm, &iter,
+                      GNUNET_GTK_SETUP_HOSTLIST_URL_MC_EDITABLE, &editable,
+                      -1);
   if (TRUE == editable)
     return FALSE;               /* likely currently editing... */
   gtk_list_store_remove (ls, &iter);
@@ -640,14 +643,18 @@ load_string_list_store (const void *cls, const char *section,
   if (ls == NULL)
     return GNUNET_SYSERR;
   t = GNUNET_strdup (value);
-  w = strtok (t, " ");
-  while (w != NULL)
-  {
-    gtk_list_store_insert_with_values (ls, &iter, G_MAXINT, 0, w, 1, FALSE, -1);
-    w = strtok (NULL, " ");
-  }
+  for (w = strtok (t, " "); w != NULL; w = strtok (NULL, " "))
+    gtk_list_store_insert_with_values (ls, &iter, G_MAXINT,
+                                       GNUNET_GTK_SETUP_HOSTLIST_URL_MC_URL, w,
+                                       GNUNET_GTK_SETUP_HOSTLIST_URL_MC_EDITABLE,
+                                       FALSE,
+                                       -1);
   GNUNET_free (t);
-  gtk_list_store_insert_with_values (ls, &iter, G_MAXINT, 0, "", 1, TRUE, -1);
+  gtk_list_store_insert_with_values (ls, &iter, G_MAXINT,
+                                     GNUNET_GTK_SETUP_HOSTLIST_URL_MC_URL, "",
+                                     GNUNET_GTK_SETUP_HOSTLIST_URL_MC_EDITABLE,
+                                     TRUE,
+                                     -1);
   return GNUNET_OK;
 }
 
@@ -682,16 +689,20 @@ save_string_list_store (const void *cls, const char *section,
   {
     do
     {
-      gtk_tree_model_get (tm, &iter, 0, &val, -1);
-      if (value == NULL)
+      gtk_tree_model_get (tm, &iter,
+                          GNUNET_GTK_SETUP_HOSTLIST_URL_MC_URL, &val, -1);
+      if (0 < strlen (val))
       {
-        value = GNUNET_strdup (val);
-      }
-      else
-      {
-        GNUNET_asprintf (&n, "%s %s", value, val);
-        GNUNET_free (value);
-        value = n;
+	if (value == NULL)
+	{
+	  value = GNUNET_strdup (val);
+	}
+	else
+	{
+	  GNUNET_asprintf (&n, "%s %s", value, val);
+	  GNUNET_free (value);
+	  value = n;
+	}
       }
       g_free (val);
     }
@@ -726,9 +737,16 @@ add_dns_entry_to_list_store (void *cls, const char *section)
 
   if (NULL == section)
   {
-    gtk_list_store_insert_with_values (ls, &iter, G_MAXINT, 0, "", 1,
-                                       (guint) 80, 2, (guint) 8080, 3,
-                                       "localhost4", 4, "tcp", -1);
+    gtk_list_store_insert_with_values (ls, &iter, G_MAXINT,
+                                       GNUNET_GTK_SETUP_GNS_MC_HOSTNAME, "",
+                                       GNUNET_GTK_SETUP_GNS_MC_SOURCEPORT,
+                                       (guint) 80,
+                                       GNUNET_GTK_SETUP_GNS_MC_TARGETPORT,
+                                       (guint) 8080,
+                                       GNUNET_GTK_SETUP_GNS_MC_HOSTNAME,
+                                       "localhost4",
+                                       GNUNET_GTK_SETUP_GNS_MC_ISUDP, "tcp",
+                                       -1);
     return;
   }
 
@@ -782,9 +800,16 @@ add_dns_entry_to_list_store (void *cls, const char *section)
                       _("`%s' is not a valid port number!\n"), hostport);
           continue;
         }
-        gtk_list_store_insert_with_values (ls, &iter, 0, 0, sld, 1,
-                                           (guint) local_port, 2,
-                                           (guint) host_port, 3, hostname, 4,
+        gtk_list_store_insert_with_values (ls, &iter, 0,
+                                           GNUNET_GTK_SETUP_GNS_MC_HOSTNAME,
+                                           sld,
+                                           GNUNET_GTK_SETUP_GNS_MC_SOURCEPORT,
+                                           (guint) local_port,
+                                           GNUNET_GTK_SETUP_GNS_MC_TARGETPORT,
+                                           (guint) host_port,
+                                           GNUNET_GTK_SETUP_GNS_MC_TARGETHOSTNAME,
+                                           hostname,
+                                           GNUNET_GTK_SETUP_GNS_MC_ISUDP,
                                            (TRUE == udp) ? "udp" : "tcp", -1);
       }
       GNUNET_free (cpy);
@@ -917,8 +942,13 @@ update_vpn_dns_configuration (GtkTreeModel * tm)
   if (TRUE == gtk_tree_model_get_iter_first (tm, &iter))
     do
     {
-      gtk_tree_model_get (tm, &iter, 0, &hostname, 1, &srcport, 2, &targetport,
-                          3, &targethost, 4, &tcpudp, -1);
+      gtk_tree_model_get (tm, &iter,
+                          GNUNET_GTK_SETUP_GNS_MC_HOSTNAME, &hostname,
+                          GNUNET_GTK_SETUP_GNS_MC_SOURCEPORT, &srcport,
+                          GNUNET_GTK_SETUP_GNS_MC_TARGETPORT, &targetport,
+                          GNUNET_GTK_SETUP_GNS_MC_TARGETHOSTNAME, &targethost,
+                          GNUNET_GTK_SETUP_GNS_MC_ISUDP, &tcpudp,
+                          -1);
       if (0 != strlen (hostname))
       {
         pos = head;
@@ -1026,7 +1056,7 @@ save_vpn_dns_service_dnsname (GtkCellRendererText * renderer, gchar * path,
     GNUNET_break (0);
     return;
   }
-  gtk_tree_model_get (tm, &iter, 0, &old, -1);
+  gtk_tree_model_get (tm, &iter, GNUNET_GTK_SETUP_GNS_MC_HOSTNAME, &old, -1);
   if ((0 != strlen (old)) && (0 == strlen (new_text)))
   {
     /* deletion */
@@ -1037,7 +1067,8 @@ save_vpn_dns_service_dnsname (GtkCellRendererText * renderer, gchar * path,
     return;
   }
   /* update model */
-  gtk_list_store_set (ls, &iter, 0, new_text, -1);
+  gtk_list_store_set (ls, &iter,
+                      GNUNET_GTK_SETUP_GNS_MC_HOSTNAME, new_text, -1);
   /* update configuration */
   update_vpn_dns_configuration (tm);
   if ((0 == strlen (old)) && (0 != strlen (new_text)))
@@ -1127,7 +1158,7 @@ save_vpn_dns_service_tcpudp (GtkCellRendererText * renderer, gchar * path,
     return;
   }
   /* update model */
-  gtk_list_store_set (ls, &iter, 4, new_text, -1);
+  gtk_list_store_set (ls, &iter, GNUNET_GTK_SETUP_GNS_MC_ISUDP, new_text, -1);
   /* update configuration */
   update_vpn_dns_configuration (tm);
 }
@@ -1211,7 +1242,8 @@ save_vpn_dns_service_sourceport (GtkCellRendererText * renderer, gchar * path,
     return;
   }
   /* update model */
-  gtk_list_store_set (ls, &iter, 1, (guint) port, -1);
+  gtk_list_store_set (ls, &iter,
+                      GNUNET_GTK_SETUP_GNS_MC_SOURCEPORT, (guint) port, -1);
   /* update configuration */
   update_vpn_dns_configuration (tm);
 }
@@ -1290,7 +1322,8 @@ save_vpn_dns_service_targethostname (GtkCellRendererText * renderer,
     return;
   }
   /* update model */
-  gtk_list_store_set (ls, &iter, 3, new_text, -1);
+  gtk_list_store_set (ls, &iter,
+                      GNUNET_GTK_SETUP_GNS_MC_TARGETHOSTNAME, new_text, -1);
   /* update configuration */
   update_vpn_dns_configuration (tm);
 }
@@ -1377,7 +1410,6 @@ static struct GNUNET_SETUP_VisibilitySpecification
 };
 
 
-
 /**
  * Hide "fs tab" if FS not active.
  */
@@ -1387,7 +1419,6 @@ static struct GNUNET_SETUP_VisibilitySpecification hide_fs_tab[] = {
 };
 
 
-
 /**
  * Hide "vpn tab" if VPN not active.
  */
@@ -1397,6 +1428,14 @@ static struct GNUNET_SETUP_VisibilitySpecification hide_vpn_tab[] = {
 };
 
 
+/**
+ * Hide "gns tab" if GNS not active.
+ */
+static struct GNUNET_SETUP_VisibilitySpecification hide_gns_tab[] = {
+  {"GNUNET_setup_gns_vbox", "(^| )gns($| )", NULL},
+  {NULL, NULL, NULL}
+};
+
 
 /**
  * Hide "tcp tab" if TCP not active.
@@ -1700,6 +1739,17 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    NULL, NULL,
    NULL},
 
+  {
+   "GNUNET_setup_hostlist_url_liststore",
+   "row-deleted",
+   "hostlist",
+   "SERVERS",
+   NULL, NULL,
+   NULL,
+   &save_string_list_store, NULL,
+   NULL, NULL,
+   NULL},
+
   {
    "GNUNET_setup_hostlist_url_treeview",
    NULL, NULL, NULL,            /* FIXME */
@@ -2491,7 +2541,19 @@ const struct GNUNET_SETUP_OptionSpecification option_specifications[] = {
    NULL, NULL,
    NULL},
 
-  /* DNS treeview */
+  /* GNS treeview */
+
+  {
+   "GNUNET_setup_general_services_gns_checkbutton",
+   "toggled",
+   "arm",
+   "DEFAULTSERVICES",
+   gettext_noop ("Should the GNS be started automatically on startup?"),
+   "https://gnunet.org/configuration-gns",
+   &load_option_list,
+   &save_option_list, "gns",
+   NULL, NULL,
+   hide_gns_tab},
 
   {
    "GNUNET_setup_gns_treeview",
