@@ -295,7 +295,7 @@ cleanup_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   if (NULL != resolver)
   {
     GNUNET_break (0 == GNUNET_OS_process_kill (resolver, SIGTERM));
-    GNUNET_OS_process_close (resolver);
+    GNUNET_OS_process_destroy (resolver);
     resolver = NULL;
   }
 }
@@ -324,6 +324,12 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   GtkWidget *main_window;
 
   ml = cls;
+
+  if (GNUNET_OK != GNUNET_GTK_main_loop_build_window (ml, NULL))
+  {
+    return;
+  }
+
   cfgName = GNUNET_GTK_main_loop_get_configuration_file (ml);
   cfg = GNUNET_CONFIGURATION_create ();
   (void) GNUNET_CONFIGURATION_load (cfg, cfgName);
