@@ -32,6 +32,56 @@
 #include <gladeui/glade.h>
 #include <gtk/gtk.h>
 
+
+/**
+ * Columns in the gns setup model.
+ */
+enum GNUNET_GTK_SETUP_HostlistUrlModelColumns
+  {
+    /**
+     * A gchararray 
+     */
+    GNUNET_GTK_SETUP_HOSTLIST_URL_MC_URL = 0,
+
+    /**
+     * A gboolean
+     */
+    GNUNET_GTK_SETUP_HOSTLIST_URL_MC_EDITABLE = 1,
+  };
+
+
+/**
+ * Columns in the hostlist url setup model.
+ */
+enum GNUNET_GTK_SETUP_GnsModelColumns
+  {
+    /**
+     * A gchararray 
+     */
+    GNUNET_GTK_SETUP_GNS_MC_HOSTNAME = 0,
+      
+    /**
+     * A guint
+     */
+    GNUNET_GTK_SETUP_GNS_MC_SOURCEPORT = 1,
+
+    /**
+     * A guint
+     */
+    GNUNET_GTK_SETUP_GNS_MC_TARGETPORT = 2,
+
+    /**
+     * A gchararray 
+     */
+    GNUNET_GTK_SETUP_GNS_MC_TARGETHOSTNAME = 3,
+
+    /**
+     * A gchararray 
+     */
+    GNUNET_GTK_SETUP_GNS_MC_ISUDP = 4,
+  };
+
+
 /**
  * Get an object from the main window.
  *
